@@ -1,8 +1,10 @@
+use crate::error::IoErrorKind;
 use crate::{
     client::{
         tls::{MaybeTlsStream, TlsPreloginWrapper},
-        AuthMethod, Config,
+        AuthMethod, Config, TrustConfig,
     },
+    error::Error,
     tds::{
         codec::{
             self, Encode, LoginMessage, Packet, PacketCodec, PacketHeader, PacketStatus,
@@ -14,7 +16,7 @@ use crate::{
     EncryptionLevel, SqlReadBytes,
 };
 #[cfg(not(any(target_os = "macos", target_os = "ios")))]
-use async_native_tls::TlsConnector;
+use async_native_tls::{Certificate, TlsConnector};
 use asynchronous_codec::Framed;
 use bytes::BytesMut;
 #[cfg(any(windows, feature = "integrated-auth-gssapi"))]
@@ -28,11 +30,11 @@ use libgssapi::{
     oid::{OidSet, GSS_MECH_KRB5, GSS_NT_KRB5_PRINCIPAL},
 };
 #[cfg(any(target_os = "macos", target_os = "ios"))]
-use opentls::async_io::TlsConnector;
+use opentls::{async_io::TlsConnector, Certificate};
 use pretty_hex::*;
 #[cfg(all(unix, feature = "integrated-auth-gssapi"))]
 use std::ops::Deref;
-use std::{cmp, fmt::Debug, io, pin::Pin, task};
+use std::{cmp, fmt::Debug, fs, io, pin::Pin, task};
 use task::Poll;
 use tracing::{event, Level};
 #[cfg(all(windows, feature = "winauth"))]
@@ -89,9 +91,7 @@ impl<S: AsyncRead + AsyncWrite + Unpin + Send> Connection<S> {
         let prelogin = connection.prelogin(config.encryption).await?;
         let encryption = prelogin.negotiated_encryption(config.encryption);
 
-        let connection = connection
-            .tls_handshake(&config, encryption, config.trust_cert)
-            .await?;
+        let connection = connection.tls_handshake(&config, encryption).await?;
 
         let mut connection = connection
             .login(
@@ -171,8 +171,7 @@ impl<S: AsyncRead + AsyncWrite + Unpin + Send> Connection<S> {
             self.transport.send(packet).await?;
         }
 
-        // Rai rai says the turbofish goodbye
-        SinkExt::<Packet>::flush(&mut self.transport).await?;
+        self.transport.flush().await?;
 
         Ok(())
     }
@@ -256,7 +255,7 @@ impl<S: AsyncRead + AsyncWrite + Unpin + Send> Connection<S> {
         }
 
         match auth {
-            #[cfg(windows)]
+            #[cfg(all(windows, feature = "winauth"))]
             AuthMethod::Integrated => {
                 let mut client = NtlmSspiBuilder::new()
                     .target_spn(self.context.spn())
@@ -376,22 +375,52 @@ impl<S: AsyncRead + AsyncWrite + Unpin + Send> Connection<S> {
         self,
         config: &Config,
         encryption: EncryptionLevel,
-        trust_cert: bool,
     ) -> crate::Result<Self> {
         if encryption != EncryptionLevel::NotSupported {
             event!(Level::INFO, "Performing a TLS handshake");
 
             let mut builder = TlsConnector::new();
 
-            if trust_cert {
-                event!(
-                    Level::WARN,
-                    "Trusting the server certificate without validation."
-                );
+            match &config.trust {
+                TrustConfig::CaCertificateLocation(path) => {
+                    if let Ok(buf) = fs::read(path) {
+                        let cert = match path.extension() {
+                            Some(ext)
+                                if ext.to_ascii_lowercase() == "pem"
+                                    || ext.to_ascii_lowercase() == "crt" =>
+                            {
+                                Some(Certificate::from_pem(&buf)?)
+                            }
+                            Some(ext) if ext.to_ascii_lowercase() == "der" => {
+                                Some(Certificate::from_der(&buf)?)
+                            }
+                            Some(_) | None => return Err(Error::Io {
+                                kind: IoErrorKind::InvalidInput,
+                                message: "Provided CA certificate with unsupported file-extension! Supported types are pem, crt and der.".to_string()}),
+                        };
+                        if let Some(c) = cert {
+                            builder = builder.add_root_certificate(c);
+                        }
+                    } else {
+                        return Err(Error::Io {
+                            kind: IoErrorKind::InvalidData,
+                            message: "Could not read provided CA certificate!".to_string(),
+                        });
+                    }
+                }
+                TrustConfig::TrustAll => {
+                    event!(
+                        Level::WARN,
+                        "Trusting the server certificate without validation."
+                    );
 
-                builder = builder.danger_accept_invalid_certs(true);
-                builder = builder.danger_accept_invalid_hostnames(true);
-                builder = builder.use_sni(false);
+                    builder = builder.danger_accept_invalid_certs(true);
+                    builder = builder.danger_accept_invalid_hostnames(true);
+                    builder = builder.use_sni(false);
+                }
+                TrustConfig::Default => {
+                    event!(Level::INFO, "Using default trust configuration.");
+                }
             }
 
             let Self {
